namespace StudySystemClient {
	public class ActivitiesView : Gtk.Box {
		private Client client;
		private Gtk.FlowBox card_container;

		public ActivitiesView(Client client)  {
			margin_top = margin_bottom = margin_start = margin_end = 0;
			this.client = client;

			var scrolled_window = new Gtk.ScrolledWindow();
			scrolled_window.hscrollbar_policy = Gtk.PolicyType.NEVER;
			scrolled_window.vexpand = true;
			scrolled_window.add_css_class("card-container");

			card_container = new Gtk.FlowBox();
			card_container.homogeneous = true;
			card_container.min_children_per_line = 1;
			card_container.max_children_per_line = 1;
			card_container.selection_mode = Gtk.SelectionMode.NONE;
			card_container.valign = Gtk.Align.START;
			scrolled_window.set_child(card_container);

			this.append(scrolled_window);

			refresh.begin((obj, res) => {
					refresh.end(res);
				});
		}

		private async void refresh() {
			try {
				var activities = yield client.list_activities();
				card_container.remove_all();
				foreach (var activity in activities) {
					var card = new ActivityCard(activity);
					card.session_logged.connect(log_session);
					card_container.append(card);
				}
			} catch (ClientError e) {
				stderr.printf("Error refreshing activities: %s\n",
							  e.message);
			}
		}

		private async void log_session(string subject, ActivityType type,
									   int minutes) {
			try {
				yield client.log_session(subject, type, minutes);
				yield refresh();
			} catch (ClientError e) {
				stderr.printf("Error logging session: %s\n", e.message);
			}
		}
	}

	private class ActivityCard : Gtk.Frame {
		public signal void session_logged(string subject, ActivityType type,
										  int minutes);

		public ActivityCard(Activity activity) {
			add_css_class("card");

			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 12);

			var text = new Gtk.Box(Gtk.Orientation.VERTICAL, 6);
			text.hexpand = true;

			var subject = new Gtk.Label(activity.subject);
			subject.halign = Gtk.Align.START;
			subject.add_css_class("activity-subject");
			text.append(subject);

			var type = new Gtk.Label(activity.type.to_string());
			type.halign = Gtk.Align.START;
			text.append(type);

			content.append(text);

			var button
				= new Gtk.Button.from_icon_name("appointment-new-symbolic");
			button.vexpand = false;
			button.valign = Gtk.Align.CENTER;
			button.set_tooltip_text("Log session");
			button.add_css_class("log-session-button");
			content.append(button);

			set_child(content);

			var log_session_popover = new LogSessionPopover();
			log_session_popover.set_parent(button);
			button.clicked.connect(() => log_session_popover.popup());
			log_session_popover.session_logged.connect((minutes) => {
					session_logged(activity.subject, activity.type, minutes);
				});
		}
	}

	private class LogSessionPopover : Gtk.Popover {
		public signal void session_logged(int minutes);

		private const int DEFAULT_LENGTH = 30;

		private Gtk.SpinButton input;

		public LogSessionPopover() {
			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);

			var label = new Gtk.Label("Minutes");
			label.halign = Gtk.Align.START;
			content.append(label);

			var adjustment
				= new Gtk.Adjustment(DEFAULT_LENGTH, 10, 480, 10, 10, 0);
			input = new Gtk.SpinButton(adjustment, 1, 0);
			input.numeric = true;
			content.append(input);

			var button = new Gtk.Button.from_icon_name("emblem-ok-symbolic");
			button.halign = Gtk.Align.END;
			button.set_tooltip_text("Submit");
			button.add_css_class("suggested-action");
			button.clicked.connect(submit);
			content.append(button);

			set_child(content);

			closed.connect(reset);
		}

		private void submit() {
			session_logged((int)input.value);
			reset();
			popdown();
		}

		private void reset() {
			input.value = DEFAULT_LENGTH;
		}
	}
}
